 #include "prettyicon.h"
 
 class PrettyIcon::Private
 {
    public:
        QPixmap bestestIcon;
        TaskPtr task;
        int currentSize;
        int scaledIconSize;
        int baseIconSize;
 };
 
PrettyIcon::PrettyIcon(TaskPtr task, int scaledIconSize, int baseIconSize, QGraphicsItem *parent)
    : Plasma::Widget(parent),
    d(new PrettyIcon::Private)
{
    d->bestestIcon = task->icon(MAX_SIZE, MAX_SIZE, false);
    d->task = task;
    d->scaledIconSize = scaledIconSize;
    d->baseIconSize = baseIconSize;
    d->currentSize = baseIconSize;
    connect(d->task.data(), SIGNAL(changed()), this, SLOT(taskChanged()));
    setAcceptsHoverEvents(true);
}

PrettyIcon::~PrettyIcon()
{
    for(int i = MIN_SIZE; i <= MAX_SIZE; i++) {
        QPixmapCache::remove(QString("prettyicons:%1:%2").arg(d->task->window()).arg(i));
    }
    delete d;
}

qreal PrettyIcon::heightForWidth(qreal w) const
{
    Q_UNUSED(w);
    return d->scaledIconSize;
}

QSizeF PrettyIcon::sizeHint() const
{
    return QSizeF(d->currentSize, d->scaledIconSize);
}

void PrettyIcon::mousePressEvent(QGraphicsSceneMouseEvent *e)
{
    switch(e->button()) {
    case Qt::LeftButton:
        if(!d->task->isActive()) {
            d->task->activate();
        } else {
            d->task->setIconified(true);
        }
        break;
    case Qt::MidButton:
        d->task->close();
        break;
    default:
        Plasma::Widget::mousePressEvent(e);
    }
}

void PrettyIcon::scaleTo(int size)
{
    d->currentSize = size;
    //kDebug() << "set to size " << size;
    update();
}

int PrettyIcon::getSize()
{
    return d->currentSize;
}

void PrettyIcon::taskChanged()
{
    update();
}

void PrettyIcon::hoverMoveEvent(QGraphicsSceneHoverEvent* e)
{
    //kDebug() << "hover move";
    if(e->pos().y() > d->scaledIconSize - d->currentSize) {
        emit gotHover(this, (float) (e->pos().x() / d->scaledIconSize));
    }
    Plasma::Widget::hoverMoveEvent(e);
}

void PrettyIcon::paintWidget(QPainter *p, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    Q_UNUSED(widget);
    //kDebug() << "painting with size " << d->currentSize;
    
    // Caching
    QPixmap pixmap(d->currentSize, d->currentSize);
    QString key = QString("prettyicons:%1:%2").arg(d->task->window()).arg(d->currentSize);
    if(!QPixmapCache::find(key, pixmap)) {
        pixmap = d->bestestIcon.scaled(d->currentSize, d->currentSize, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
        QPixmapCache::insert(key, pixmap);
    }
    p->drawPixmap(0, option->exposedRect.height() - d->currentSize, d->currentSize, d->currentSize, pixmap);
    if(d->task->isMinimized()) {
        p->setBrush(QBrush(Qt::white));
        QPolygon triangle;
        int middleBottom = option->exposedRect.width() / 2;
        int height = option->exposedRect.height() - 2;
        triangle.putPoints(0, 3, middleBottom - 5, height, middleBottom, height - 5, middleBottom + 5, height);
        p->drawPolygon(triangle, Qt::OddEvenFill);
    }
}
