#include "prettylayout.h"

class PrettyLayout::Private
{
    public:
        Private()
            : columnWidth( -1 )
        {};
        QList<Plasma::LayoutItem*> items;
        qreal columnWidth;
        bool hasHover;
        PrettyTasks* parent;
};

PrettyLayout::PrettyLayout(PrettyTasks* parent)
    : Layout(parent),
    d(new Private)
{
    d->parent = parent;
}

PrettyLayout::~PrettyLayout()
{
    releaseManagedItems();
    delete d;
}

int PrettyLayout::count() const
{
    return d->items.count();
}

void PrettyLayout::addItem(Plasma::LayoutItem* item)
{
    if (!item || d->items.contains(item)) {
        return;
    }

    item->setManagingLayout(this);
    d->items << item;

    if (animator()) {
        animator()->setCurrentState(item,Plasma::LayoutAnimator::InsertedState);
    }

    updateGeometry();
}
void PrettyLayout::removeItem(Plasma::LayoutItem* item)
{
    if (!item) {
        return;
    }

    item->unsetManagingLayout(this);
    d->items.removeAll(item);

    if (animator()) {
        animator()->setCurrentState(item,Plasma::LayoutAnimator::RemovedState);
    }

    updateGeometry();
}
int PrettyLayout::indexOf(Plasma::LayoutItem* item) const
{
    if (!item) {
        return -1;
    }

    return d->items.indexOf(item);
}
Plasma::LayoutItem* PrettyLayout::itemAt(int i) const
{
    if (i >= d->items.count()) {
        return 0;
    }

    return d->items[i];
}

QSizeF PrettyLayout::sizeHint() const
{
    return QSizeF(calculateMaxWidth(), d->parent->calculateSize(0, 0, 0.5));
}

int PrettyLayout::calculateMaxWidth() const
{
    const int count = d->items.count();
    const int middle = count / 2;
    const qreal space = spacing();
    int width = space;
    for(int i = 0; i < count; i++) {
        width += d->parent->calculateSize(i, middle, 0.5) + space;
    }
    return width;
}

Plasma::LayoutItem* PrettyLayout::takeAt(int i)
{
    if (i >= d->items.count()) {
        return 0;
    }

    return d->items.takeAt(i);
    // FIXME: Should updateGeometry() be called?
}

void PrettyLayout::relayout()
{
    const QRectF rect = /*adjustToMargins(*/geometry()/*)*/;
    const qreal space = spacing();
    
    int allItemWidth = space;
    
    foreach(Plasma::LayoutItem *item , d->items) {
        allItemWidth += dynamic_cast<PrettyIcon*>(item)->getSize() + space;
    }
    
    int currentPosition = (rect.width() - allItemWidth) / 2;
    int itemSize = 0;
    
    foreach(Plasma::LayoutItem *item , d->items) {
        currentPosition += space;
        //kDebug() << currentPosition << " is currentPosition";
        itemSize = dynamic_cast<PrettyIcon*>(item)->getSize();
        const QRectF newGeometry(rect.left() + currentPosition,
                                 rect.top(),
                                 itemSize,
                                 rect.height());
        if(animator()){
            animator()->setGeometry(item, newGeometry);
        } else {
            item->setGeometry(newGeometry);
        }
        currentPosition += itemSize;
    }

    startAnimation();
}

void PrettyLayout::releaseManagedItems()
{
    foreach (Plasma::LayoutItem *item, d->items) {
        item->unsetManagingLayout(this);
    }
}

Qt::Orientations PrettyLayout::expandingDirections() const
{
    return Qt::Horizontal;
}
